package uk.plc.graphic.classes;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import uk.plc.graphic.database.DatabaseConnection;

public class Error {

    // --------------------------- Attributes ------------------------------------
    private int id;
    private String message;
    private Timestamp timestamp;

    public Error(int id, String message, Timestamp timestamp) {
        this.id = id;
        this.message = message;
        this.timestamp = timestamp;
    }

    // ------------------------ Accessor Methods ---------------------------------
    public int getId() {
        return id;
    }

    public String getMessage() {
        return message;
    }

    public Timestamp getTimestamp() {
        return timestamp;
    }

    public void setId(int id) {
        this.id = id;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public void setTimestamp(Timestamp timestamp) {
        this.timestamp = timestamp;
    }

    // ---------------------- Data Access Methods --------------------------------
    public static Error getError(int id) throws SQLException{
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `errors` WHERE id = ?");
            preparedStatement.setInt(1, id);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                String message = result.getString("message");
                Timestamp timestamp = result.getTimestamp("timestamp");
                return new Error(id, message, timestamp);
            }
            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static Error insertError(String message) throws SQLException{
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("INSERT INTO `errors` VALUES(NULL, ?, ?");
            preparedStatement.setString(1, message);
            Timestamp timestamp = new Timestamp(System.currentTimeMillis());
            preparedStatement.setTimestamp(2, timestamp);
            preparedStatement.execute();
            int id = DatabaseConnection.getLastInsertIdInt(conn);
            return new Error(id, message, timestamp);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void editError() throws SQLException{
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();

            // Edit the title.
            preparedStatement = conn.prepareStatement("UPDATE `errors` SET `message`=? WHERE id=?");
            preparedStatement.setString(1, message);
            preparedStatement.setInt(2, id);
            preparedStatement.execute();

            // Edit the number of columns.
            preparedStatement = conn.prepareStatement("UPDATE `errors` SET `timestamp`=? WHERE id=?");
            preparedStatement.setTimestamp(1, timestamp);
            preparedStatement.setInt(2, id);
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void deleteError() throws SQLException{
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("DELETE FROM `errors` WHERE id=?");
            preparedStatement.setInt(1, id);
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }
}
