/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package uk.plc.graphic.classes;

import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Vector;

/**
 *
 * @author Max
 */
public abstract class Folder<T extends FolderItem> {
    private Vector<T> files;
    private HashMap<String, Folder> folders;
    private Folder parent;
    private String name;

    public Folder(String name) {
        this.name = name;
        this.files = new Vector<T>();
        this.folders = new HashMap<String, Folder>();
    }

    public void add(Folder<T> folder) {
        folder.setParent(this);
        this.folders.put(folder.getName(), folder);
    }

    public void add(T file) {
        this.files.add(file);
    }

    public Folder<T> getFolderByUri(String uri) {
        try {
            String[] folderArray = uri.split("/");
            Folder currentFolder = this;
            Folder newFolder;
            for (String currentFolderName : folderArray) {
                if (currentFolderName.equals("")) {
                    continue;

                }
                if (currentFolder.containsFolder(currentFolderName)) {
                    currentFolder = currentFolder.getFolder(currentFolderName);
                } else {
                    newFolder = this.getClass().getConstructor(new Class[]{String.class}).newInstance(new Object[]{currentFolderName});
                    currentFolder.add(newFolder);
                    currentFolder = newFolder;
                }
            }
            return currentFolder;
            //none of the following exceptions should happen
        } catch (NoSuchMethodException noSuchMethodException) {
        } catch (SecurityException securityException) {
        } catch (InstantiationException instantiationException) {
        } catch (IllegalAccessException illegalAccessException) {
        } catch (IllegalArgumentException illegalArgumentException) {
        } catch (InvocationTargetException invocationTargetException) {
        }
        return null;
    }

    public Folder<T> getFolder(String foldername) {
        return this.folders.get(foldername);
    }

    public boolean containsFolder(String foldername) {
        return folders.containsKey(foldername);
    }

    public Folder<T>[] getFolders() {
        Vector<Folder> folderVec = new Vector<Folder>();
        folderVec.addAll(folders.values());
        Collections.sort(folderVec, new Comparator<Folder>() {

            public int compare(Folder o1, Folder o2) {
                return o1.getName().compareTo(o2.getName());
            }
        });
        return folderVec.toArray(new Folder[folderVec.size()]);
    }

    public Folder<T>[] getAllFolders() {
        Vector<Folder> folderVec = new Vector<Folder>();
        Folder[] subs;
        if (this.getParent() == null) {
            folderVec.add(this);

        }
        for (Folder folder : folders.values()) {
            folderVec.add(folder);
            subs = folder.getAllFolders();
            for (Folder f : subs) {
                folderVec.add(f);

            }
        }
        Collections.sort(folderVec, new Comparator<Folder>() {

            public int compare(Folder o1, Folder o2) {
                return o1.getFullUri().compareTo(o2.getFullUri());
            }
        });
        return folderVec.toArray(new Folder[folderVec.size()]);
    }

    public FolderItem[] getFiles() {
        Collections.sort(files, new Comparator<T>() {
            public int compare(T o1, T o2) {
                return o1.getFilename().compareTo(o2.getFilename());
            }
        });
        return this.files.toArray(new FolderItem[this.files.size()]);
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    public String getFullUri() {
        Folder parentFolder = getParent();
        StringBuffer buffer = new StringBuffer();
        buffer.append(getName() + "/");
        while (parentFolder != null) {
            buffer.insert(0, parentFolder.getName() + "/");
            parentFolder = parentFolder.getParent();
        }
        return buffer.toString();
    }

    /**
     * @return the parent
     */
    public Folder getParent() {
        return parent;
    }

    /**
     * @param parent the parent to set
     */
    public void setParent(Folder parent) {
        this.parent = parent;
    }
}
