package uk.plc.graphic.classes;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import uk.plc.graphic.database.DatabaseConnection;

public class NewsItem {

  // --------------------------- Attributes ------------------------------------
  private int id;
  private String text;
  private Timestamp timestamp;
  private String headline;

  public NewsItem(int id, String text, Timestamp timestamp, String headline) {
    this.id = id;
    this.text = text;
    this.timestamp = timestamp;
    this.headline = headline;
  }

  // ------------------------ Accessor Methods ---------------------------------
  public int getId() {
    return id;
  }

  public String getText() {
    return text;
  }

  public Timestamp getTimestamp() {
    return timestamp;
  }

  public String getHeadline() {
    return headline;
  }

  public void setId(int id) {
    this.id = id;
  }

  public void setText(String text) {
    this.text = text;
  }

  public void setTimestamp(Timestamp timestamp) {
    this.timestamp = timestamp;
  }

  public void setHeadline(String headline) {
    this.headline = headline;
  }

  // ---------------------- Data Access Methods --------------------------------
  public static List<NewsItem> listNewsItems() throws SQLException {
    Connection conn = null;
    ResultSet result = null;
    PreparedStatement preparedStatement = null;
    try {
      conn = DatabaseConnection.getConnection();
      preparedStatement = conn.prepareStatement("SELECT * FROM `news_items`");
      result = preparedStatement.executeQuery();
      List<NewsItem> newsItems = new ArrayList<NewsItem>();
      while (result.next()) {
        int id = result.getInt("id");
        String text = result.getString("text");
        Timestamp timestamp = result.getTimestamp("timestamp");
        String headline = result.getString("headline");
        newsItems.add(new NewsItem(id, text, timestamp, headline));
      }
      return newsItems;
    } finally {
      DatabaseConnection.closeConnection(conn);
      DatabaseConnection.closeResultSet(result);
      DatabaseConnection.closePreparedStatement(preparedStatement);
    }
  }

    public static List<NewsItem> listRecentNewsItems() throws SQLException {
    Connection conn = null;
    ResultSet result = null;
    PreparedStatement preparedStatement = null;
    try {
      conn = DatabaseConnection.getConnection();
      preparedStatement = conn.prepareStatement("SELECT * FROM `news_items` ORDER BY timestamp DESC LIMIT 3");
      result = preparedStatement.executeQuery();
      List<NewsItem> newsItems = new ArrayList<NewsItem>();
      while (result.next()) {
        int id = result.getInt("id");
        String text = result.getString("text");
        Timestamp timestamp = result.getTimestamp("timestamp");
        String headline = result.getString("headline");
        newsItems.add(new NewsItem(id, text, timestamp, headline));
      }
      return newsItems;
    } finally {
      DatabaseConnection.closeConnection(conn);
      DatabaseConnection.closeResultSet(result);
      DatabaseConnection.closePreparedStatement(preparedStatement);
    }
  }

  public static NewsItem getNewsItem(int id) throws SQLException {
    Connection conn = null;
    PreparedStatement preparedStatement = null;
    ResultSet result = null;
    try {
      conn = DatabaseConnection.getConnection();
      preparedStatement = conn.prepareStatement("SELECT * FROM `news_items` WHERE id = ?");
      preparedStatement.setInt(1, id);
      result = preparedStatement.executeQuery();
      if (result.next()) {
        String text = result.getString("text");
        Timestamp timestamp = result.getTimestamp("timestamp");
        String headline = result.getString("headline");
        return new NewsItem(id, text, timestamp, headline);
      }
      return null;
    } finally {
      DatabaseConnection.closePreparedStatement(preparedStatement);
      DatabaseConnection.closeResultSet(result);
      DatabaseConnection.closeConnection(conn);
    }
  }

  public static NewsItem insertNewsItem(String text, String headline) throws SQLException {
    Connection conn = null;
    PreparedStatement preparedStatement = null;
    try {
      conn = DatabaseConnection.getConnection();
      preparedStatement = conn.prepareStatement("INSERT INTO `news_items` VALUES(NULL, ?, ?, ?)");
      preparedStatement.setString(1, text);
      Timestamp timestamp = new Timestamp(System.currentTimeMillis());
      preparedStatement.setTimestamp(2, timestamp);
      preparedStatement.setString(3, headline);
      preparedStatement.execute();
      int id = DatabaseConnection.getLastInsertIdInt(conn);
      return new NewsItem(id, text, timestamp, headline);
    } finally {
      DatabaseConnection.closePreparedStatement(preparedStatement);
      DatabaseConnection.closeConnection(conn);
    }
  }

  public void editNewsItem() throws SQLException {
    Connection conn = null;
    PreparedStatement preparedStatement = null;
    try {
      conn = DatabaseConnection.getConnection();

      // Edit the title.
      preparedStatement = conn.prepareStatement("UPDATE `news_items` SET `text`=? WHERE id=?");
      preparedStatement.setString(1, text);
      preparedStatement.setInt(2, id);
      preparedStatement.execute();

      // Edit the number of columns.
      preparedStatement = conn.prepareStatement("UPDATE `news_items` SET `timestamp`=? WHERE id=?");
      preparedStatement.setTimestamp(1, timestamp);
      preparedStatement.setInt(2, id);
      preparedStatement.execute();

      preparedStatement = conn.prepareStatement("UPDATE `news_items` SET `headline`=? WHERE id=?");
      preparedStatement.setString(1, headline);
      preparedStatement.setInt(2, id);
      preparedStatement.execute();
    } finally {
      DatabaseConnection.closePreparedStatement(preparedStatement);
      DatabaseConnection.closeConnection(conn);
    }
  }

  public void delete() throws SQLException {
    Connection conn = null;
    PreparedStatement preparedStatement = null;
    try {
      conn = DatabaseConnection.getConnection();
      preparedStatement = conn.prepareStatement("DELETE FROM `news_items` WHERE id=?");
      preparedStatement.setInt(1, id);
      preparedStatement.execute();
    } finally {
      DatabaseConnection.closePreparedStatement(preparedStatement);
      DatabaseConnection.closeConnection(conn);
    }
  }
}
