package uk.plc.graphic.classes;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import uk.plc.graphic.database.DatabaseConnection;

/**
 *
 * @author Max
 */
public class TextBlock {

    private int id;
    private int column;
    private int position;
    private String text;
    private String key;
    private String title;

    public int getId() {
        return id;
    }

    public String getText() {
        return text;
    }

    public void setId(int id) {
        this.id = id;
    }

    public void setText(String text) {
        this.text = text;
    }

    public TextBlock(int id, String text, String key, String title, int column, int position) {
        this.id = id;
        this.text = text;
        this.key = key;
        this.title = title;
        this.column = column;
        this.position = position;
    }

    public static TextBlock getTextBlock(int id) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `textblocks` WHERE id = ?");
            preparedStatement.setInt(1, id);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                String text = result.getString("data");
                String key = result.getString("key");
                String title = result.getString("title");
                int position = result.getInt("column");
                int column = result.getInt("position");
                return new TextBlock(id, text, key, title, column, position);
            }
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
        return null;
    }

    public static TextBlock insertTextBlock(String text, String key, String title, 
            int column, int position) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("INSERT INTO `textblocks` (`key`, `title`, `data`, `column`, `position`)"
                    + " VALUES (?, ?, ?, ?, ?)");
            preparedStatement.setString(1, key);
            preparedStatement.setString(2, title);
            preparedStatement.setString(3, text);
            preparedStatement.setInt(4, column);
            preparedStatement.setInt(5, position);
            preparedStatement.execute();
            int id = DatabaseConnection.getLastInsertIdInt(conn);
            return new TextBlock(id, text, key, title, column, position);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }

    }

    public void editTextBlock() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("UPDATE `textblocks` SET `data`=?, `title`=? WHERE id=?");
            preparedStatement.setString(1, text);
            preparedStatement.setString(2, title);
            preparedStatement.setInt(3, id);
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }

    }

    public void deleteTextBlock() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("DELETE FROM `textblocks` WHERE id=?");
            preparedStatement.setInt(1, id);
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    /**
     * @return the key
     */
    public String getKey() {
        return key;
    }

    /**
     * @param key the key to set
     */
    public void setKey(String key) {
        this.key = key;
    }

    /**
     * @return the title
     */
    public String getTitle() {
        return title;
    }

    /**
     * @param title the title to set
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * @return the column
     */
    public int getColumn() {
        return column;
    }

    /**
     * @return the position
     */
    public int getPosition() {
        return position;
    }
}
