package uk.plc.graphic.classes;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.Vector;
import uk.plc.graphic.database.DatabaseConnection;

public class UploadedFile implements FolderItem{

    private String uri;
    private String filename;
    private String contentType;
    private long filesize;
    private int id;
    private Date added;
    private Date edited;
    private InputStream data;
    private final static int BUFFER_SIZE = 1 << 8; //1KiB buffer

    public UploadedFile(int id, String uri, String filename, String contentType,
            long filesize, Date added, Date edited) {
        this.uri = uri;
        this.filename = filename;
        this.contentType = contentType;
        this.id = id;
        this.filesize = filesize;
        this.added = added;
        this.edited = edited;
    }

    public boolean isImage() {
        return (this instanceof UploadedImage);
    }

    public static UploadedFile getFile(int id) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            String sql = "SELECT `uri`, `filename`, `content-type`, `added`, `edited`, `filesize`, `width`, `height` FROM `files` WHERE id = ?";
            preparedStatement = conn.prepareStatement(sql);
            preparedStatement.setInt(1, id);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                String uri = result.getString("uri");
                String filename = result.getString("filename");
                String contentType = result.getString("content-type");
                boolean isImage = (UploadedImage.isContentTypeImage(contentType));
                long addedTimestamp = result.getLong("added");
                long editedTimestamp = result.getLong("edited");
                int filesize = result.getInt("filesize");
                if (isImage) {
                    int width = result.getInt("width");
                    int height = result.getInt("height");
                    return new UploadedImage(id, uri, filename, contentType, filesize,
                            new Date(addedTimestamp), new Date(editedTimestamp), width, height);
                } else {
                    return new UploadedFile(id, uri, filename, contentType, filesize,
                            new Date(addedTimestamp), new Date(editedTimestamp));
                }
            }
            return null;
//        }catch(NullPointerException ex){
//            //MYSQL bug: result set is being closed
//            StackTraceElement found = null;
//            StackTraceElement el;
//            StackTraceElement[] elArray = ex.getStackTrace();
//            for(int i=0; i<elArray.length&&found==null; i++ ){
//                el = elArray[i];
//                if(el.getClassName().matches("^uk\\.plc\\.graphic\\..*$")){
//                    found = el;
//                }
//            }
//            throw new SQLException("MySQL connector bug, result set was closed "+
//                    (found!=null?"("+found.getClassName()+"."+
//                    found.getMethodName()+"["+found.getLineNumber()+"])":""));
////            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static UploadedFile getFileByURI(String uri) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            String sql = "SELECT `id`, `filename`, `content-type`, `added`, `edited`, `filesize`, `width`, `height` FROM `files` WHERE uri = ?";
            preparedStatement = conn.prepareStatement(sql);
            preparedStatement.setString(1, uri);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                int id = result.getInt("id");
                String filename = result.getString("filename");
                String contentType = result.getString("content-type");
                boolean isImage = (UploadedImage.isContentTypeImage(contentType));
                long addedTimestamp = result.getLong("added");
                long editedTimestamp = result.getLong("edited");
                long filesize = result.getLong("filesize");
                if (isImage) {
                    int width = result.getInt("width");
                    int height = result.getInt("height");
                    return new UploadedImage(id, uri, filename, contentType, filesize,
                            new Date(addedTimestamp), new Date(editedTimestamp), width, height);
                } else {
                    return new UploadedFile(id, uri, filename, contentType, filesize,
                            new Date(addedTimestamp), new Date(editedTimestamp));
                }
            }
            return null;
//        }catch(NullPointerException ex){
//            //MYSQL bug: result set is being closed
//            StackTraceElement found = null;
//            StackTraceElement el;
//            StackTraceElement[] elArray = ex.getStackTrace();
//            for(int i=0; i<elArray.length&&found==null; i++ ){
//                el = elArray[i];
//                if(el.getClassName().matches("^uk\\.plc\\.graphic\\..*$")){
//                    found = el;
//                }
//            }
//            throw new SQLException("MySQL connector bug, result set was closed "+
//                    (found!=null?"("+found.getClassName()+"."+
//                    found.getMethodName()+"["+found.getLineNumber()+"])":""));
////            return null;
        }finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static boolean uriExists(String uri) throws SQLException {
        uri = UploadedFile.cleanUri(uri);
        return (UploadedFile.getFileByURI(uri) != null);
    }

    public static UploadedFile insertFile(String uri, String filename, String contentType,
            long filesize, byte[] data) throws SQLException {
        Date added = new Date();
        Date edited = added;
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            filename = UploadedFile.cleanFilename(filename);
            uri = UploadedFile.cleanUri(uri);
            boolean isImage = UploadedImage.isContentTypeImage(contentType);
            String sql;
            if (isImage) {
                sql = "INSERT INTO `files` "
                        + "(`uri`,`filename`,`content-type`, `data`, `filesize`, `added`, `edited`, `height`, `width`)"
                        + " VALUES(?,?, ?, ?, ?, ?, ?, ?, ?)";
            } else {
                sql = "INSERT INTO `files` "
                        + "(`uri`,`filename`,`content-type`, `data`, `filesize`, `added`, `edited`)"
                        + " VALUES(?,?, ?, ?, ?, ?, ?)";
            }
            preparedStatement = conn.prepareStatement(sql);
            preparedStatement.setString(1, uri);
            preparedStatement.setString(2, filename);
            preparedStatement.setString(3, contentType);
//            Blob blob = conn.createBlob();
//            blob.setBytes(1, data);
//            preparedStatement.setBlob(4, blob);
            preparedStatement.setBytes(4, data);
//            File out = new File("out.png");
//            FileOutputStream fo = null;
//            try{
//                fo = new FileOutputStream(out);
//                fo.write(data);
//            }catch(IOException ex){
//
//            }finally{
//                try{if(fo!=null)fo.close();}catch(IOException ex){}
//            }
            preparedStatement.setLong(5, filesize);
            preparedStatement.setLong(6, added.getTime());
            preparedStatement.setLong(7, edited.getTime());
            if (isImage) {
                java.awt.Image img = UploadedImage.getImageFromData(data);
                int tries = 100;
                int width = -1;
                int height = -1;
                while ((width < 0 || height < 0) && tries > 0) {
                    width = img.getWidth(null);
                    height = img.getHeight(null);
                    //System.out.println("Try: "+tries+" w: "+width+" h: "+height);
                    tries--;
                    //Prevent stack overflow
                    try {
                        Thread.sleep(1);
                    } catch (InterruptedException ex) {
                    }
                }
                preparedStatement.setInt(8, height);
                preparedStatement.setInt(9, width);
            }
//            System.out.println(" debuggable statement= " + preparedStatement.toString());

            preparedStatement.execute();
            int id = DatabaseConnection.getLastInsertIdInt(conn);
            return new UploadedFile(id, uri, filename, contentType, filesize, added, edited);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }

    }

//

    public static FileFolder getSortedFiles() throws SQLException {
        UploadedFile[] files = UploadedFile.getFiles();
        FileFolder folder = new FileFolder("");
        String folderUri;
        FileFolder filesFolder;
        for (UploadedFile file : files) {
            folderUri = file.getFolderUri();
            filesFolder = folder.getFolderByUri(folderUri);
            filesFolder.add(file);
        }
        return folder;
    }

    public static UploadedFile[] getFiles() throws SQLException {
        Vector<UploadedFile> files = new Vector<UploadedFile>();
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `files` ORDER BY `uri`, `filename` ASC");
            result = preparedStatement.executeQuery();
            int id;
            String filename;
            String uri;
            String contentType;
            boolean isImage;
            long addedTimestamp;
            long editedTimestamp;
            long filesize;
            int width;
            int height;
            while (result.next()) {
                id = result.getInt("id");
                uri = result.getString("uri");
                filename = result.getString("filename");
                contentType = result.getString("content-type");
                isImage = (UploadedImage.isContentTypeImage(contentType));
                addedTimestamp = result.getLong("added");
                editedTimestamp = result.getLong("edited");
                filesize = result.getLong("filesize");
                if (isImage) {
                    width = result.getInt("width");
                    height = result.getInt("height");
                    files.add(new UploadedImage(id, uri, filename, contentType, filesize,
                            new Date(addedTimestamp), new Date(editedTimestamp), width, height));
                } else {
                    files.add(new UploadedFile(id, uri, filename, contentType, filesize,
                            new Date(addedTimestamp), new Date(editedTimestamp)));
                }
            }
            return files.toArray(new UploadedFile[files.size()]);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void editFile() throws SQLException {
        edited = new Date();
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();

            // Edit the title.
            preparedStatement = conn.prepareStatement("UPDATE `files` "
                    + "SET `uri`=?, `filename`=?, `edited`=? WHERE id=?");
            preparedStatement.setString(1, getUri());
            preparedStatement.setString(2, getFilename());
            preparedStatement.setLong(3, getEdited().getTime());
            preparedStatement.setInt(4, getId());
            preparedStatement.execute();

        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void deleteFile() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("DELETE FROM `files` WHERE id=?");
            preparedStatement.setInt(1, getId());
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public InputStream getData() throws SQLException {
        if (data != null) {
            return data;
        }
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT data FROM `files` where id= ? ");
            preparedStatement.setInt(1, this.getId());
            result = preparedStatement.executeQuery();
            if (result.next()) {
                Blob blob = result.getBlob("data");
                data = blob.getBinaryStream();
            }
            return data;
//        }catch(NullPointerException ex){
//            //MYSQL bug: result set is being closed
//            StackTraceElement found = null;
//            StackTraceElement el;
//            StackTraceElement[] elArray = ex.getStackTrace();
//            for(int i=0; i<elArray.length&&found==null; i++ ){
//                el = elArray[i];
//                if(el.getClassName().matches("^uk\\.plc\\.graphic\\..*$")){
//                    found = el;
//                }
//            }
//            throw new SQLException("MySQL connector bug, result set was closed "+
//                    (found!=null?"("+found.getClassName()+"."+
//                    found.getMethodName()+"["+found.getLineNumber()+"])":""));
////            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public byte[] getBytes() throws IOException, SQLException {
        return UploadedFile.getBytes(getData());
    }

    public void write(OutputStream out) throws IOException, SQLException {
        UploadedFile.write(getData(), out);
    }

    public static byte[] getBytes(InputStream stream) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            byte[] buffer = new byte[BUFFER_SIZE];
            int bytesRead = -1;
            while ((bytesRead = stream.read(buffer)) > -1) {
                out.write(buffer, 0, bytesRead);
            }
            byte[] bytes = out.toByteArray();
            return bytes;
        } finally {
            stream.close();
            out.close();
        }
    }

    public static void write(InputStream in, OutputStream out) throws IOException {
        try {
            byte[] buffer = new byte[BUFFER_SIZE];
            int bytesRead = -1;
            while ((bytesRead = in.read(buffer)) > -1) {
                out.write(buffer, 0, bytesRead);
            }
        } finally {
            in.close();
            out.close();
        }
    }

    public static String cleanFilename(String filename) {
        if (filename.matches("^[a-zA-Z0-9 _\\-\\.\\[\\]]*$")) {
            return filename;

        }
        return filename.replaceAll("[^a-zA-Z0-9 _\\-\\.\\[\\]]", "");
    }

    public static String cleanUri(String uri) {
        if (uri.matches("^[a-zA-Z0-9 _\\-\\.\\/\\[\\]]*$")) {
            return uri;

        }
        return uri.replaceAll("[^a-zA-Z0-9 _\\-\\.\\/\\[\\]]", "");
    }

    /**
     * @return the uri
     */
    public String getUri() {
        return uri;
    }

    /**
     * @return the uri
     */
    public String getFolderUri() {
        return uri.substring(0, uri.lastIndexOf(filename));
    }

    /**
     * @param uri the uri to set
     */
    public void setUri(String uri) {
        uri = UploadedFile.cleanUri(uri);
        this.uri = uri;
    }

    /**
     * @return the filename
     */
    public String getFilename() {
        return filename;
    }

    /**
     * @param filename the filename to set
     */
    public void setFilename(String filename) {
        filename = UploadedFile.cleanFilename(filename);
        this.filename = filename;
    }

    /**
     * @return the contentType
     */
    public String getContentType() {
        return contentType;
    }

    /**
     * @return the filesize
     */
    public long getFilesize() {
        return filesize;
    }

    /**
     * @return the id
     */
    public int getId() {
        return id;
    }

    /**
     * @return the added
     */
    public Date getAdded() {
        return added;
    }

    /**
     * @return the edited
     */
    public Date getEdited() {
        return edited;
    }




}
