package uk.plc.graphic.classes;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import uk.plc.graphic.database.DatabaseConnection;

public class User {

    private int id;
    private String name;
    private String email;

    public User(int id, String name, String email) {
        this.id = id;
        this.name = name;
        this.email = email;
    }

    public int getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getEmail() {
        return email;
    }

    public void setId(int id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public static User login(String username, String password) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `users` WHERE email = ? AND password = MD5(?)");
            preparedStatement.setString(1, username);
            preparedStatement.setString(2, password);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                int id = result.getInt("id");
                String name = result.getString("name");
                String email = result.getString("email");
                return new User(id, name, email);
            }
            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static User getUser(int id) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        ResultSet result = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("SELECT * FROM `users` WHERE id = ?");
            preparedStatement.setInt(1, id);
            result = preparedStatement.executeQuery();
            if (result.next()) {
                String name = result.getString("name");
                String email = result.getString("email");
                return new User(id, name, email);
            }
            return null;
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeResultSet(result);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public static User insertUser(String name, String email, String password) throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("INSERT INTO `users` VALUES(?,?,?)");
            preparedStatement.setString(1, name);
            preparedStatement.setString(1, email);
            preparedStatement.setString(1, password);
            preparedStatement.execute();
            int id = DatabaseConnection.getLastInsertIdInt(conn);
            return new User(id, name, email);
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void editUser() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();

            // Edit the name.
            preparedStatement = conn.prepareStatement("UPDATE `users` SET `name`=? WHERE id=?");
            preparedStatement.setString(1, name);
            preparedStatement.setInt(2, id);
            preparedStatement.execute();

            // Edit the email address.
            preparedStatement = conn.prepareStatement("UPDATE `users` SET `email`=? WHERE id=?");
            preparedStatement.setString(1, email);
            preparedStatement.setInt(2, id);
            preparedStatement.execute();

        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }

    public void deleteUser() throws SQLException {
        Connection conn = null;
        PreparedStatement preparedStatement = null;
        try {
            conn = DatabaseConnection.getConnection();
            preparedStatement = conn.prepareStatement("DELETE FROM `users` WHERE id=?");
            preparedStatement.setInt(1, id);
            preparedStatement.execute();
        } finally {
            DatabaseConnection.closePreparedStatement(preparedStatement);
            DatabaseConnection.closeConnection(conn);
        }
    }
}
