/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uk.plc.graphic.filters;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URLEncoder;
import java.sql.SQLException;
import java.util.Enumeration;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import uk.plc.graphic.classes.NewsItem;

/**
 *
 * @author djw213
 */
public class GetNewsByIdentifier implements Filter {

  private static final boolean debug = true;
  // The filter configuration object we are associated with.  If
  // this value is null, this filter instance is not currently
  // configured.
  private FilterConfig filterConfig = null;

  public GetNewsByIdentifier() {
  }

  /**
   *
   * @param request The servlet request we are processing
   * @param response The servlet response we are creating
   * @param chain The filter chain we are processing
   *
   * @exception IOException if an input/output error occurs
   * @exception ServletException if a servlet error occurs
   */
  public void doFilter(ServletRequest request, ServletResponse response,
          FilterChain chain)
          throws IOException, ServletException {

    Enumeration names = request.getAttributeNames();
    String namesStr = "";
    while (names.hasMoreElements()) {
      namesStr += (String)names.nextElement() + ",";
    }

    boolean gotNewsItem = false;
    String error = null;
    if (request instanceof HttpServletRequest) {
      if (request.getParameter("n") != null) {
        try {
          int identifier = Integer.parseInt(request.getParameter("n"));
          NewsItem newsItem = NewsItem.getNewsItem(identifier);

          if (newsItem != null) {
            request.setAttribute("news_item_to_edit", newsItem);
            gotNewsItem = true;
          } else {
            error = "News item with ID " + identifier + " not found";
            request.setAttribute("error", error);
          }
        } catch (NumberFormatException ex) {
          error = "News item ID is not a valid number";
          request.setAttribute("error", error);
        } catch (SQLException ex) {
          error = "SQL Error: " + ex.getMessage();
          request.setAttribute("error", error);
        }
      } else {
        // We can assume that we are creating a new news item.
        gotNewsItem = true;
      }
    }

    if (gotNewsItem) {
      chain.doFilter(request, response);
      return;
    } else {
      if (response instanceof HttpServletResponse) {
        ((HttpServletResponse) response).sendRedirect("/admin/news-listing.jsp?err="
                + (error != null ? "?err=" + URLEncoder.encode(error, "UTF-8") : ""));
                //URLEncoder.encode((String) request.getAttribute("error"), "UTF-8"));
      }
    }
  }

  /**
   * Return the filter configuration object for this filter.
   */
  public FilterConfig getFilterConfig() {
    return (this.filterConfig);
  }

  /**
   * Set the filter configuration object for this filter.
   *
   * @param filterConfig The filter configuration object
   */
  public void setFilterConfig(FilterConfig filterConfig) {
    this.filterConfig = filterConfig;
  }

  /**
   * Destroy method for this filter
   */
  public void destroy() {
  }

  /**
   * Init method for this filter
   */
  public void init(FilterConfig filterConfig) {
    this.filterConfig = filterConfig;
    if (filterConfig != null) {
      if (debug) {
        log("GetNewsByIdentifier:Initializing filter");
      }
    }
  }

  /**
   * Return a String representation of this object.
   */
  @Override
  public String toString() {
    if (filterConfig == null) {
      return ("GetNewsByIdentifier()");
    }
    StringBuffer sb = new StringBuffer("GetNewsByIdentifier(");
    sb.append(filterConfig);
    sb.append(")");
    return (sb.toString());
  }

  public static String getStackTrace(Throwable t) {
    String stackTrace = null;
    try {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      t.printStackTrace(pw);
      pw.close();
      sw.close();
      stackTrace = sw.getBuffer().toString();
    } catch (Exception ex) {
    }
    return stackTrace;
  }

  public void log(String msg) {
    filterConfig.getServletContext().log(msg);
  }
}
