package uk.plc.graphic.filters;

import java.io.IOException;
import java.util.List;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import uk.plc.graphic.classes.NewsItem;

/**
 *
 * @author David Walker
 */
public class GetNewsItems implements Filter {

  private static final boolean DEBUG = true;
  private FilterConfig filterConfig = null;

  public GetNewsItems() {
  }

  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
    try {
      if (request.getParameter("err") != null) {
        request.setAttribute("error", request.getParameter("err"));
      }

      StringBuffer listing = new StringBuffer();
      List<NewsItem> newsItems = NewsItem.listNewsItems();
      listing.append(getNewsItemListing(newsItems));

      request.setAttribute("no_news_items", newsItems.size());
      request.setAttribute("news_listing", listing.toString());
    } catch (Exception ex) {
      request.setAttribute("error", "SQL Error "+ex.toString());
    }
    chain.doFilter(request, response);
  }

  /**
   * Return the filter configuration object for this filter.
   */
  public FilterConfig getFilterConfig() {
    return (this.filterConfig);
  }

  /**
   * Set the filter configuration object for this filter.
   *
   * @param filterConfig The filter configuration object
   */
  public void setFilterConfig(FilterConfig filterConfig) {
    this.filterConfig = filterConfig;
  }

  public static String getNewsItemListing(List<NewsItem> newsItems) {
    StringBuffer buffer = new StringBuffer("<ul class=\"folderlisting\">");
    for (NewsItem newsItem : newsItems) {
      buffer.append("<li class=\"page\">");

      buffer.append("<table cellpadding=\"0\" cellpadding=\"0\" style=\"display:inline;\">");
      buffer.append("<tr>");
      buffer.append("<td>");
      buffer.append(newsItem.getHeadline());
      buffer.append("</td>");
      buffer.append("<td>");
      buffer.append("<a href=\"/admin/edit-news.jsp?n=" + newsItem.getId() + "\" class=\"edit\" title=\"Edit\">&nbsp;<!--edit--></a>");
      buffer.append("</td>");
      buffer.append("</tr>");
      buffer.append("</table>");
      buffer.append("</li>");
    }
    buffer.append("</ul>");
    return buffer.toString();
  }

  /**
   * Destroy method for this filter
   */
  public void destroy() {
  }

  /**
   * Init method for this filter
   */
  public void init(FilterConfig filterConfig) {
    this.filterConfig = filterConfig;
    if (filterConfig != null) {
      if (DEBUG) {
        log("GetNewsItems:Initializing filter");
      }
    }
  }

  /**
   * Return a String representation of this object.
   */
  @Override
  public String toString() {
    if (filterConfig == null) {
      return ("GetNewsItems()");
    }
    StringBuffer sb = new StringBuffer("GetNewsItems(");
    sb.append(filterConfig);
    sb.append(")");
    return (sb.toString());
  }

  public void log(String msg) {
    filterConfig.getServletContext().log(msg);
  }
}
