/*
        Sets up the skeleton framework for the ajax query, 
                e.g.
                var ajaxhandler = new AjaxHandler();
                ajaxhandler.create("json", 4000, "json");
 
                var queue = new AjaxQueue(ajaxhandler.ajax_objs["json"]); or var queue = new AjaxQueue(ajaxhandler.ajax_objs.json);
                queue.run();
 
                queue.addQuery("json.html?i="+index, "GET", null, result);
 
 */


function Request(){ //holds the timelimit and the actual xmlhttp object
    this.currCacheID = -1;
    this.requestTimer = null;
    if(window.ActiveXObject){
        //microsoft
        try {
            this.xmlHttp = new ActiveXObject("Msxml2.XMLHTTP");
        } catch (e) {
            try {
                this.xmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
            } catch (e) {}
        }
    }else if(window.XMLHttpRequest){
        //mozilla opera etc
        this.xmlHttp = new XMLHttpRequest();
        //needed for some versions of mozilla
        if(this.type == "xml") this.xmlHttp.overrideMimeType(this.type);
    }
    if (!this.xmlHttp) {
        alert('Giving up :( Cannot create an XMLHTTP instance');
        return false;
    }
    return true;
}

var AjaxHandler = function AjaxHandler(){ }; // used to make sure there are at most 2 xmlHttp requests 
AjaxHandler.ajax_objs = new Array();
AjaxHandler.count = 0;
AjaxHandler.create = function(name, time, t){ 
    if(this.count >= 2){
        alert("Too many Ajax objects, maximum of 2 are allowed"); 
        return null;
    }else if(typeof(this.ajax_objs[name]) != "undefined"){
        alert("Ajax object with that name already exists"); 
        return null;
    }else{
        var ajax = new Ajax(time, t);
        this.count++;
        this.ajax_objs[name] = ajax;
        return ajax;
    //debug("created ajax object: "+name, "#000000");
    }
};
AjaxHandler.destroy = function(name){
    this.ajax_objs[name].destroy();
    this.ajax_objs[name] = null;
    this.count--;
    
};

Ajax.prototype.request = new Request();
Ajax.prototype.constructor=Ajax;




function Ajax(time, t){  // the main ajax engine
    
    //ajax_count++; 
    this.max_time = (!time)? 4000 : time;
    this.type = (!t)? "xml" : t;
    this.testQuery = "/assets/js/test-conn.html";
//this.failTimeout = null;
} 

Ajax.prototype.destroy = function(){
    this.request = null;
    this.max_time = null;
    this.type = null;
};

Ajax.prototype.runQuery = function(testConn, testConnTimeout){ // Runs the queryURL, method is get or post, if get then post should be null 
    if(!testConn) var testConn = false;
    if(!testConnTimeout) var testConnTimeout = this.max_time;
    var obj = this; //needed to deal with the timeout not allowing "this" to be used to reference the object (ends up referencing the window.)
    var stamp = new Date();
    var cacheid = stamp.valueOf(); //Math.round(100*Math.random()); // to prevent caching
    
    var queryURL;
    var method;
    var post;
    
    if(!testConn){
        queryURL = AjaxQueue.currQuery.queryURL;
        method = AjaxQueue.currQuery.method;
        post = AjaxQueue.currQuery.post;
    }else{
        queryURL = this.testQuery;
        method = "GET";
        post = "";
    }
    
    queryURL += (queryURL.search(/\?/g) == -1)? "?cacheid="+cacheid : "&cacheid="+cacheid;
    this.request.currCacheID = cacheid;
    this.request.xmlHttp.open(method, queryURL, true);
    if(!testConn){
        
        this.request.xmlHttp.onreadystatechange = function(){
            obj.aj_handler();
        };
        this.request.requestTimer = setTimeout(function(){
            obj.aj_abort();
        }, this.max_time);
    }else{
        Utils.debug("I've Sent for test connection: "+this.request.currCacheID);
        this.request.xmlHttp.onreadystatechange = function(){
            obj.failedHandler();
        };
        this.request.requestTimer = setTimeout(function(){
            obj.testConnectionAbort();
        }, testConnTimeout);
    }
    //debug(queryURL);
    
    
    //TIMEOUT FUNCTION
    
    
    
    if(post != null) this.request.xmlHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    if(post == null) post = "";
    
    this.request.xmlHttp.send(post);
    
};

Ajax.prototype.failedHandler = function(){
    
    if (this.request.xmlHttp.readyState == 4) {
        
        var object = null;
        try{
            if (this.request.xmlHttp.status == 200){
                Utils.debug("I've reached the failed handler: "+this.request.currCacheID);
                clearTimeout(this.request.requestTimer); //clears the abort timer
                if(AjaxQueue.currQuery.retry){
                    AjaxQueue.run();
                }else{
                    AjaxQueue.next(); //loop queue
                    AjaxQueue.run();
                }
                var abortFunc = AjaxQueue.currQuery.abortFunc;
                if(typeof(abortFunc)=="function") abortFunc(false);
            }
        }catch(e){
        }
    }
};
Ajax.prototype.testConnectionAbort = function(){
    var abortFunc = AjaxQueue.currQuery.abortFunc;
    Utils.debug("I've reached test connection aborted: "+this.request.currCacheID);
    this.request.xmlHttp.abort();
    AjaxQueue.next(); //loop queue
    AjaxQueue.run();
    if(typeof(abortFunc)=="function") abortFunc(true);
};
Ajax.prototype.failed = function(){
    var self = this;
    setTimeout(function(){
        self.runQuery(true, 10000);
    }, 1000);
};



Ajax.prototype.aj_abort = function (){
    Utils.debug("I've Aborted: "+this.request.currCacheID);
    this.request.xmlHttp.abort(); //aborts the request
    this.failed();
    
};

Ajax.prototype.aj_handler = function (){ //handles the result of the query
    if (this.request.xmlHttp.readyState == 4) {
        
        var object = null;
        try{
            if (this.request.xmlHttp.status == 200)  {
                clearTimeout(this.request.requestTimer); //clears the abort timer
                
                if(this.request.xmlHttp.responseText != ""){
                    if(this.type == "json"){
                        
                        var json_text = this.request.xmlHttp.responseText.replace("�", "\xA3"); //deals with the � sign
                        
                        try{
							
                            object = (typeof(parseJSON) == "function")?json_text.parseJSON() : eval("(" + json_text + ")");	/*if the json 
                                              parser is available, use that instead of the unsafe eval function*/
                        }catch(e){
                            var abortFunc = AjaxQueue.currQuery.abortFunc;
                            if(typeof(abortFunc)=="function") abortFunc(false);
                            AjaxQueue.loop();
                            return;
                        }
                    }else if(this.type == "xml"){
                        if(window.ActiveXObject){
                            //microsoft
                            var xmlDocument = new ActiveXObject("Microsoft.XMLDOM");
                            xmlDocument.loadXML(this.request.xmlHttp.responseText);
                            object = xmlDocument.documentElement; // deal with microsofts dodgy xml handling
                        }else{
                            object = this.request.xmlHttp.responseXML.documentElement;
                        }
                    }else if(this.type == "plain"){
                        object = this.request.xmlHttp.responseText;
                    }else if(this.type == "javascript"){
                        object = eval(this.request.xmlHttp.responseText);
                    }else if(this.type == "html"){
                        var trimmed = this.request.xmlHttp.responseText.replace(/^\s*/, "").replace(/\s*$/, "");
                        var dummyLayer = Utils.newElement("div",
							{'id': "dummy"});
                        dummyLayer.innerHTML = trimmed;
                        object = dummyLayer.firstChild;
                    }
                }
                var func = AjaxQueue.currQuery.func;
                
                var param = AjaxQueue.currQuery.param;
                if(typeof(func) === 'function'){
                    if(param == null){	
                        
                        func(object); //if the return function exists pass the new object to it 
                    }else{
                        
                        func(object, param);
                    }
                }
                AjaxQueue.loop(); //loop queue
            } else {
                AjaxQueue.loop(); //loop queue
            }
        }catch(e){
        //debug("Exception Caught: "+e);
            
        }
        
    } 
};

Ajax.prototype.redeclare= function(time, t){
    this.max_time = (!time)? 4000 : time;
    this.type = (!t)? "xml" : t;
}



var AjaxQueue = function(){};
AjaxQueue.queue = new Array();
AjaxQueue.runTimeout = null;

//function to add a query to the queue, method is get or post, if get then post should be null. Func is called with the returned object as a parameter.
AjaxQueue.addQuery = function(ajaxObj, queryURL, method, post, func, param, abortFunc, rtry){
    if(!rtry) var rtry = false;
    this.queue.push( {
        "ajaxObj" : ajaxObj,
        "queryURL" : queryURL,
        "method": method,
        "post" : post,
        "func": func,
        "param":param,
        "abortFunc" : abortFunc,
        "retry": rtry
    });
}; 


AjaxQueue.run = function(){
    var query = this.queue[0];
    if(query != null){
        if(this.runTimeout != null) clearTimeout(this.runTimeout); //clears the running timeout to stop stacking while trying to run a query
        //debug("About To Run Query: "+query.queryURL+" Method: "+query.method+" Param: "+query.param, "#000000");
        this.currQuery =query;
        query.ajaxObj.runQuery(); //runs the query
    }else{
        var self = this;
        this.runTimeout = setTimeout(function(){
            self.run();
        }, 50); // loops until it finds a query to run
    }
};
AjaxQueue.next = function(){
    if (this.queue[0] != null) {
        this.queue.shift();
    }
};
AjaxQueue.loop = function(){
    if (this.queue[0] != null) {
        this.queue.shift(); // shifts the array to run the next query
        
        this.run();
    }
};